@@ -353,7 +353,7 @@ void test_match_cmd(void){
   if( zGap==0 ) zGap = " ... ";
   p = search_init(g.argv[2], zBegin, zEnd, zGap, flg);
   for(i=3; i<g.argc; i++){
-    blob_read_from_file(&x, g.argv[i]);
+    blob_read_from_file(&x, g.argv[i], ExtFILE);
     zDoc = blob_str(&x);
     score = search_match(p, 1, (const char**)&zDoc);
     fossil_print("%s: %d\n", g.argv[i], p->iScore);
@@ -532,22 +532,23 @@ static void search_urlencode_sqlfunc(
 */
 void search_sql_setup(sqlite3 *db){
   static int once = 0;
+  static const int enc = SQLITE_UTF8|SQLITE_INNOCUOUS;
   if( once++ ) return;
-  sqlite3_create_function(db, "search_match", -1, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "search_match", -1, enc, 0,
      search_match_sqlfunc, 0, 0);
-  sqlite3_create_function(db, "search_score", 0, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "search_score", 0, enc, 0,
      search_score_sqlfunc, 0, 0);
-  sqlite3_create_function(db, "search_snippet", 0, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "search_snippet", 0, enc, 0,
      search_snippet_sqlfunc, 0, 0);
-  sqlite3_create_function(db, "search_init", -1, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "search_init", -1, enc, 0,
      search_init_sqlfunc, 0, 0);
-  sqlite3_create_function(db, "stext", 3, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "stext", 3, enc, 0,
      search_stext_sqlfunc, 0, 0);
-  sqlite3_create_function(db, "title", 3, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "title", 3, enc, 0,
      search_title_sqlfunc, 0, 0);
-  sqlite3_create_function(db, "body", 3, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "body", 3, enc, 0,
      search_body_sqlfunc, 0, 0);
-  sqlite3_create_function(db, "urlencode", 1, SQLITE_UTF8, 0,
+  sqlite3_create_function(db, "urlencode", 1, enc, 0,
      search_urlencode_sqlfunc, 0, 0);
 }
 
@@ -640,7 +641,8 @@ void search_cmd(void){
 #define SRCH_TKT      0x0004    /* Search over tickets */
 #define SRCH_WIKI     0x0008    /* Search over wiki */
 #define SRCH_TECHNOTE 0x0010    /* Search over tech notes */
-#define SRCH_ALL      0x001f    /* Search over everything */
+#define SRCH_FORUM    0x0020    /* Search over forum messages */
+#define SRCH_ALL      0x003f    /* Search over everything */
 #endif
 
 /*
@@ -656,11 +658,13 @@ unsigned int search_restrict(unsigned int srchFlags){
      { SRCH_TKT,      "search-tkt"  },
      { SRCH_WIKI,     "search-wiki" },
      { SRCH_TECHNOTE, "search-technote" },
+     { SRCH_FORUM,    "search-forum" },
   };
   int i;
   if( g.perm.Read==0 )   srchFlags &= ~(SRCH_CKIN|SRCH_DOC|SRCH_TECHNOTE);
   if( g.perm.RdTkt==0 )  srchFlags &= ~(SRCH_TKT);
   if( g.perm.RdWiki==0 ) srchFlags &= ~(SRCH_WIKI);
+  if( g.perm.RdForum==0) srchFlags &= ~(SRCH_FORUM);
   for(i=0; i<count(aSetng); i++){
     unsigned int m = aSetng[i].m;
     if( (srchFlags & m)==0 ) continue;
@@ -752,7 +756,7 @@ static void search_fullscan(
       ")"
       "INSERT INTO x(label,url,score,id,date,snip)"
       "  SELECT printf('Check-in [%%.10s] on %%s',uuid,datetime(mtime)),"
-      "         printf('/timeline?c=%%s&n=8&y=ci',uuid),"
+      "         printf('/timeline?c=%%s',uuid),"
       "         search_score(),"
       "         'c'||rid,"
       "         datetime(mtime),"
@@ -795,6 +799,19 @@ static void search_fullscan(
       "   WHERE search_match('',body('e',rid,NULL));"
     );
   }
+  if( (srchFlags & SRCH_FORUM)!=0 ){
+    db_multi_exec(
+      "INSERT INTO x(label,url,score,id,date,snip)"
+      "  SELECT 'Forum '||comment,"
+      "         '/forumpost/'||uuid,"
+      "         search_score(),"
+      "         'f'||rid,"
+      "         datetime(event.mtime),"
+      "         search_snippet()"
+      "    FROM event JOIN blob on event.objid=blob.rid"
+      "   WHERE search_match('',body('f',rid,NULL));"
+    );
+  }
 }
 
 /*
@@ -891,7 +908,7 @@ static void search_indexed(
 ){
   Blob sql;
   if( srchFlags==0 ) return;
-  sqlite3_create_function(g.db, "rank", 1, SQLITE_UTF8, 0,
+  sqlite3_create_function(g.db, "rank", 1, SQLITE_UTF8|SQLITE_INNOCUOUS, 0,
      search_rank_sqlfunc, 0, 0);
   blob_init(&sql, 0, 0);
   blob_appendf(&sql,
@@ -915,6 +932,7 @@ static void search_indexed(
        { SRCH_TKT,      't' },
        { SRCH_WIKI,     'w' },
        { SRCH_TECHNOTE, 'e' },
+       { SRCH_FORUM,    'f' },
     };
     int i;
     for(i=0; i<count(aMask); i++){
@@ -954,6 +972,7 @@ static char *cleanSnippet(const char *zSnip){
   int i;
   int n = 0;
   char *z;
+  if( zSnip==0 ) zSnip = "";
   for(i=0; zSnip[i]; i++) if( zSnip[i]=='<' ) n++;
   z = fossil_malloc( i+n*4+1 );
   i = 0;
@@ -1050,16 +1069,23 @@ int search_run_and_output(
 ** permissions and the server configuration.  The entry box is shown
 ** disabled if srchFlags is 0 after these restrictions are applied.
 **
-** If useYparam is true, then this routine also looks at the y= query
-** parameter for further search restrictions.
+** The mFlags value controls options:
+**
+**     0x01    If the y= query parameter is present, use it as an addition
+**             restriction what to search.
+**
+**     0x02    Show nothing if search is disabled.
+**
+** Return true if there are search results.
 */
-void search_screen(unsigned srchFlags, int useYparam){
+int search_screen(unsigned srchFlags, int mFlags){
   const char *zType = 0;
   const char *zClass = 0;
   const char *zDisable1;
   const char *zDisable2;
   const char *zPattern;
   int fDebug = PB("debug");
+  int haveResult = 0;
   srchFlags = search_restrict(srchFlags);
   switch( srchFlags ){
     case SRCH_CKIN:     zType = " Check-ins";  zClass = "Ckin"; break;
@@ -1067,13 +1093,15 @@ void search_screen(unsigned srchFlags, int useYparam){
     case SRCH_TKT:      zType = " Tickets";    zClass = "Tkt";  break;
     case SRCH_WIKI:     zType = " Wiki";       zClass = "Wiki"; break;
     case SRCH_TECHNOTE: zType = " Tech Notes"; zClass = "Note"; break;
+    case SRCH_FORUM:    zType = " Forum";      zClass = "Frm";  break;
   }
   if( srchFlags==0 ){
+    if( mFlags & 0x02 ) return 0;
     zDisable1 = " disabled";
     zDisable2 = " disabled";
     zPattern = "";
   }else{
-    zDisable1 = " autofocus";
+    zDisable1 = ""; /* Was: " autofocus" */
     zDisable2 = "";
     zPattern = PD("s","");
   }
@@ -1084,14 +1112,15 @@ void search_screen(unsigned srchFlags, int useYparam){
     @ <div class='searchForm'>
   }
   @ <input type="text" name="s" size="40" value="%h(zPattern)"%s(zDisable1)>
-  if( useYparam && (srchFlags & (srchFlags-1))!=0 && useYparam ){
-    static const struct { char *z; char *zNm; unsigned m; } aY[] = {
+  if( (mFlags & 0x01)!=0 && (srchFlags & (srchFlags-1))!=0 ){
+    static const struct { const char *z; const char *zNm; unsigned m; } aY[] = {
        { "all",  "All",        SRCH_ALL      },
        { "c",    "Check-ins",  SRCH_CKIN     },
        { "d",    "Docs",       SRCH_DOC      },
        { "t",    "Tickets",    SRCH_TKT      },
        { "w",    "Wiki",       SRCH_WIKI     },
        { "e",    "Tech Notes", SRCH_TECHNOTE },
+       { "f",    "Forum",      SRCH_FORUM    },
     };
     const char *zY = PD("y","all");
     unsigned newFlags = srchFlags;
@@ -1128,7 +1157,9 @@ void search_screen(unsigned srchFlags, int useYparam){
       @ <p class='searchEmpty'>No matches for: <span>%h(zPattern)</span></p>
     }
     @ </div>
+    haveResult = 1;
   }
+  return haveResult;
 }
 
 /*
@@ -1144,6 +1175,7 @@ void search_screen(unsigned srchFlags, int useYparam){
 **                      t -> tickets
 **                      w -> wiki
 **                      e -> tech notes
+**                      f -> forum
 **                    all -> everything
 */
 void search_page(void){
@@ -1251,6 +1283,7 @@ static void append_all_ticket_fields(Blob *pAccum, Stmt *pQuery, int iTitle){
 **                      c      Check-in comment
 **                      t      Ticket text
 **                      e      Tech note
+**                      f      Forum
 **
 **    rid               The RID of an artifact that defines the object
 **                      being searched.
@@ -1276,13 +1309,23 @@ void search_stext(
       blob_reset(&doc);
       break;
     }
+    case 'f':     /* Forum messages */
     case 'e':     /* Tech Notes */
     case 'w': {   /* Wiki */
       Manifest *pWiki = manifest_get(rid,
-          cType == 'e' ? CFTYPE_EVENT : CFTYPE_WIKI, 0);
+          cType == 'e' ? CFTYPE_EVENT :
+          cType == 'f' ? CFTYPE_FORUM : CFTYPE_WIKI, 0);
       Blob wiki;
       if( pWiki==0 ) break;
-      blob_init(&wiki, pWiki->zWiki, -1);
+      if( cType=='f' ){
+        blob_init(&wiki, 0, 0);
+        if( pWiki->zThreadTitle ){
+          blob_appendf(&wiki, "<h1>%h</h1>\n", pWiki->zThreadTitle);
+        }
+        blob_appendf(&wiki, "From %s:\n\n%s", pWiki->zUser, pWiki->zWiki);
+      }else{
+        blob_init(&wiki, pWiki->zWiki, -1);
+      }
       get_stext_by_mimetype(&wiki, wiki_filter_mimetypes(pWiki->zMimetype),
                             pOut);
       blob_reset(&wiki);
@@ -1423,7 +1466,7 @@ void test_convert_stext(void){
   Blob in, out;
   db_find_and_open_repository(0,0);
   if( g.argc!=4 ) usage("FILENAME MIMETYPE");
-  blob_read_from_file(&in, g.argv[2]);
+  blob_read_from_file(&in, g.argv[2], ExtFILE);
   blob_init(&out, 0, 0);
   get_stext_by_mimetype(&in, g.argv[3], &out);
   fossil_print("%s\n",blob_str(&out));
@@ -1517,7 +1560,7 @@ void search_fill_index(void){
   );
   db_multi_exec(
     "INSERT OR IGNORE INTO ftsdocs(type,rid,name,idxed)"
-    "  SELECT 'e', objid, comment, 0 FROM event WHERE type='e';"
+    "  SELECT type, objid, comment, 0 FROM event WHERE type IN ('e','f');"
   );
 }
 
@@ -1554,6 +1597,7 @@ void search_doc_touch(char cType, int rid, const char *zName){
         cType, zName, rid
       );
     }
+    /* All forum posts are always indexed */
   }
 }
 
@@ -1683,6 +1727,29 @@ static void search_update_wiki_index(void){
   );
 }
 
+/*
+** Deal with all of the unindexed 'f' terms in FTSDOCS
+*/
+static void search_update_forum_index(void){
+  db_multi_exec(
+    "INSERT INTO ftsidx(docid,title,body)"
+    " SELECT rowid, title('f',rid,NULL),body('f',rid,NULL) FROM ftsdocs"
+    "  WHERE type='f' AND NOT idxed;"
+  );
+  if( db_changes()==0 ) return;
+  db_multi_exec(
+    "UPDATE ftsdocs SET idxed=1, name=NULL,"
+    " (label,url,mtime) = "
+    "  (SELECT 'Forum '||event.comment,"
+    "          '/forumpost/'||blob.uuid,"
+    "          event.mtime"
+    "     FROM event, blob"
+    "    WHERE event.objid=ftsdocs.rid"
+    "      AND blob.rid=ftsdocs.rid)"
+    "WHERE ftsdocs.type='f' AND NOT ftsdocs.idxed"
+  );
+}
+
 /*
 ** Deal with all of the unindexed 'e' terms in FTSDOCS
 */
@@ -1729,6 +1796,9 @@ void search_update_index(unsigned int srchFlags){
   if( srchFlags & SRCH_TECHNOTE ){
     search_update_technote_index();
   }
+  if( srchFlags & SRCH_FORUM ){
+    search_update_forum_index();
+  }
 }
 
 /*
@@ -1775,12 +1845,13 @@ void fts_config_cmd(void){
      { 4,  "enable"   },
      { 5,  "stemmer"  },
   };
-  static const struct { char *zSetting; char *zName; char *zSw; } aSetng[] = {
-     { "search-ckin",     "check-in search:",  "c" },
+  static const struct { const char *zSetting; const char *zName; const char *zSw; } aSetng[] = {
+     { "search-ci",       "check-in search:",  "c" },
      { "search-doc",      "document search:",  "d" },
      { "search-tkt",      "ticket search:",    "t" },
      { "search-wiki",     "wiki search:",      "w" },
      { "search-technote", "tech note search:", "e" },
+     { "search-forum",    "forum search:",     "f" },
   };
   char *zSubCmd = 0;
   int i, j, n;
